#
# fishing.R, 29 Oct 23
#
# Data from:
# Early Knight data
# https://shape-of-code.com/2016/04/30/costperformance-analysis-of-1944-1967-computers-knights-data/
# 1985 Knight data
# A functional and structural measurement of technology
# by Kenneth Knight
# ATTRIBUTES OF THE PERFORMANCE OF CENTRAL PROCESSING UNITS: A RELATIVE PERFORMANCE PREDICTION MODEL
# by PHILLIP EIN-DOR and JACOB FELDMESSER
# KOP data
# TRACKING THE ELUSIVE KOPS
# by Edward J. Lias
# Whetstone data
# http://www.roylongbottom.org.uk/whetstone.htm
# Roy Longbottom's website
# Dhrystone data
# https://netlib.org/performance/html/dhrystone.data.col0.html
# Alfred Aburto / Naval Ocean Systems Center, San Diego / aburto@nosc.mil


library("colorspace")
library("lme4")


par(bty="l")
par(las=1)
par(pch="+")
pal_col=rainbow(2)


k50=read.csv("../Benchmark-historical/knight1950s.csv", as.is=TRUE)
k50$Bench="Knight-50"
k50$metric=k50$Sci.ops.sec
k60=read.csv("../Benchmark-historical/knight1960s.csv", as.is=TRUE)
k60$Bench="Knight-60"
k60$metric=k60$Sci.ops.sec
k60$Sci.sec.dol=NA  # to match the k50 data

k5060=rbind(k50, k60)
k5060=subset(k5060, !duplicated(System))  # Merge the Knight 50/60 datasets
k5060$Bench="Knight-5060"

k85=read.csv("../Benchmark-historical/knight1985.csv", as.is=TRUE)
k85$Bench="Knight-85"
k85$metric=k85$op.sec
kop=read.csv("../Benchmark-historical/mainframe-bench-1980_KOPS.csv", as.is=TRUE)
kop$Bench="KOP"
kop$metric=kop$KOPS
ein=read.csv("../Benchmark-historical/ein-dor.csv", as.is=TRUE)
ein$Bench="ein-dor"
ein$metric=ein$performance
whet=read.csv("../Benchmark-historical/WhetRoyLongbottom.csv", as.is=TRUE)
whet$Bench="Whetstone"
whet$metric=whet$MWIPS
dhry=read.csv("../Benchmark-historical/drystone.csv", as.is=TRUE)
dhry$Bench="Dhrystone"
dhry$metric=dhry$V2.1
npl=read.csv("../Benchmark-historical/NPL-unpub.csv", as.is=TRUE)
npl$Bench="NPL"
# npl$metric=npl$???

k85op=merge(k85, kop, all=TRUE)
ke=merge(k85op, ein, all=TRUE)
kwhet=merge(ke, whet, all=TRUE)
kdhry=merge(kwhet, dhry, all=TRUE) # Not included in the analysis

k56ll=merge(kwhet, k5060, all=TRUE)

Sall=k56ll

# sort(Sall$System)

# kdhry=merge(Sall, dhry, all=TRUE)
# sort(kdhry$System)


# Plot paired benchmark results for the same System
s85op=merge(k85, kop, by="System", all=TRUE)
se=merge(s85op, ein, by="System", all=TRUE)
swhet=merge(se, whet, by="System", all=TRUE)
sdhry=merge(swhet, dhry, by="System", all=TRUE) # Not included in the analysis

s56ll=merge(swhet, k5060, by="System", all=TRUE)

sys_all=s56ll

plot(~ log(op.sec)+log(KOPS)+log(performance)+log(MWIPS), data=sys_all, cex=1.4)


# Model pairs of benchmark results

kops_sec_mod=glm(log(KOPS) ~ log(op.sec), data=Sall)
summary(kops_sec_mod)

kops_perf_mod=glm(log(KOPS) ~ log(performance), data=Sall)
summary(kops_perf_mod)

perf_mod=glm(log(performance) ~ log(1/MCYT)+CACH, data=Sall)
summary(perf_mod)

MWIPS_perf_mod=glm(log(MWIPS) ~ log(performance), data=Sall)
summary(MWIPS_perf_mod)

MWIPS_KOPS_mod=glm(log(MWIPS) ~ log(KOPS), data=Sall)
summary(MWIPS_KOPS_mod)

MWIPS_mod=glm(log(MWIPS) ~ log(Clock.MHz)+Lang, data=whet)
summary(MWIPS_mod)

# Remove non-Fortran results from Whetstone data
Sall_clean=subset(Sall, is.na(Lang) | (Lang == "For"))

nrow(Sall_clean)
length(unique(Sall_clean$System))

# ben_mod=lmer(log(metric) ~ System+(System|Bench), data=Sall)
ben_mod=lmer(log(metric) ~ (1|System)+(1|Bench), data=Sall_clean)
summary(ben_mod)

# The Intercept values are relative performance values for
# System and Bench
ranef(ben_mod)

relperf=exp(ranef(ben_mod)$System) # un-log performance result
relperf$System=rownames(relperf)
relperf$relperf=relperf$'(Intercept)'

t=merge(relperf, Sall_clean, all=TRUE)
t$Date=sub("^[0-9]+/", "19", t$Date) # strip out month

# Aggregate known years
t$Year=t$Year.intro
t$Year[is.na(t$Year)]=t$Date[is.na(t$Year)]
t$Year=as.numeric(t$Year)

plot(t$Year, t$relperf, log="y", col=pal_col[2],
	ylim=c(1e-3, 1e3),
	xlab="Year introduced", ylab="Performance (relative)")

xbounds=1960:2000

rp_mod=glm(log(relperf) ~ Year, data=t)
summary(rp_mod)

pred=predict(rp_mod, newdata=data.frame(Year=xbounds))
lines(xbounds, exp(pred), col=pal_col[1])

